@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.53 2003/07/05 11:56:20 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.61 2003/08/27 00:36:21 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -65,7 +65,7 @@ enum {
 	TICK,
 	GOT_METADATA,
 	BUFFERING,
-	LAST_SIGNAL
+	LAST_SIGNAL,
 };
 
 /* Enum for none-signal stuff that needs to go through the AsyncQueue */
@@ -74,7 +74,8 @@ enum {
 	TITLE_CHANGE_ASYNC,
 	EOS_ASYNC,
 	CHANNELS_CHANGE_ASYNC,
-	BUFFERING_ASYNC
+	BUFFERING_ASYNC,
+	MESSAGE_ASYNC,
 };
 
 typedef struct {
@@ -148,7 +149,6 @@ struct BaconVideoWidgetPrivate {
 	gboolean logo_mode;
 	guint tick_id;
 	gboolean auto_resize;
-	GdkPixbuf *icon;
 	int volume;
 	TvOutType tvout;
 	guint32 video_fcc, audio_fcc;
@@ -230,7 +230,7 @@ bacon_video_widget_get_type (void)
 		};
 
 		bacon_video_widget_type = g_type_register_static
-			(GTK_TYPE_VBOX, "BaconVideoWidget",
+			(GTK_TYPE_BOX, "BaconVideoWidget",
 			 &bacon_video_widget_info, (GTypeFlags)0);
 	}
 
@@ -247,7 +247,7 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 	object_class = (GObjectClass *) klass;
 	widget_class = (GtkWidgetClass *) klass;
 
-	parent_class = gtk_type_class (gtk_vbox_get_type ());
+	parent_class = gtk_type_class (gtk_box_get_type ());
 
 	/* GtkWidget */
 	widget_class->realize = bacon_video_widget_realize;
@@ -412,10 +412,8 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 		i++;
 	}
 
-	bvw->priv->tick_id = g_timeout_add (70,
+	bvw->priv->tick_id = g_timeout_add (140,
 			(GSourceFunc) bacon_video_widget_tick_send, bvw);
-
-	bvw->priv->icon = gdk_pixbuf_new_from_file (ICON_PATH, NULL);
 }
 
 static void
@@ -426,9 +424,6 @@ bacon_video_widget_finalize (GObject *object)
 	g_list_foreach (bvw->priv->visuals, (GFunc) g_free, NULL);
 	g_list_free (bvw->priv->visuals);
 
-	if (bvw->priv->icon != NULL)
-		gdk_pixbuf_unref (bvw->priv->icon);
-
 	/* Should put here what needs to be destroyed */
 	g_idle_remove_by_data (bvw);
 	g_async_queue_unref (bvw->priv->queue);
@@ -476,11 +471,13 @@ frame_output_cb (void *bvw_gen,
 
 	/* correct size with video_pixel_aspect */
 	if (video_pixel_aspect >= bvw->priv->display_ratio)
+	{
 		video_width = video_width * video_pixel_aspect
 			/ bvw->priv->display_ratio + .5;
-	else
+	} else {
 		video_height = video_height * bvw->priv->display_ratio
 			/ video_pixel_aspect + .5;
+	}
 
 	*dest_x = 0;
 	*dest_y = 0;
@@ -1040,9 +1037,11 @@ bacon_video_widget_realize (GtkWidget *widget)
 	setup_config_video (bvw);
 
 	if (bvw->priv->null_out == FALSE && bvw->priv->ao_driver != NULL)
+	{
 		bvw->priv->vis = xine_post_init (bvw->priv->xine,
 				bvw->priv->vis_name, 0,
 				&bvw->priv->ao_driver, &bvw->priv->vo_driver);
+	}
 
 	bacon_resize_init ();
 	bvw->priv->stream = xine_stream_new (bvw->priv->xine,
@@ -1095,6 +1094,11 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 				bvw_table_signals[BUFFERING],
 				0, data->num);
 		break;
+	case MESSAGE_ASYNC:
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[ERROR],
+				0, data->msg, TRUE);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1108,11 +1112,11 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 	return (queue_length > 0);
 }
 
-//FIXME this needs to be launched from the async handler
 static void
 xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 {
 	char *message;
+	signal_data *sigdata;
 
 	message = NULL;
 
@@ -1162,11 +1166,11 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 		return;
 	}
 
-	g_signal_emit (G_OBJECT (bvw),
-			bvw_table_signals[ERROR], 0,
-			message, TRUE);
-
-	g_free (message);
+	sigdata = g_new0 (signal_data, 1);
+	sigdata->signal = MESSAGE_ASYNC;
+	sigdata->msg = message;
+	g_async_queue_push (bvw->priv->queue, sigdata);
+	g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
 }
 
 static void
@@ -1266,6 +1270,8 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 
 	bvw = BACON_VIDEO_WIDGET (widget);
 
+	g_source_remove (bvw->priv->tick_id);
+
 	/* stop the playback */
 	xine_stop (bvw->priv->stream);
 	xine_close (bvw->priv->stream);
@@ -1344,9 +1350,12 @@ bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 	GTK_WIDGET(bvw)->requisition.width = width;
 	GTK_WIDGET(bvw)->requisition.height = height;
 
+	bvw->priv->init_width = width;
+	bvw->priv->init_height = height;
+
 	/* load the video drivers */
 	bvw->priv->ao_driver = load_audio_out_driver (bvw, err);
-	if (*err != NULL)
+	if (err != NULL && *err != NULL)
 		return NULL;
 
 	bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
@@ -1459,7 +1468,7 @@ bacon_video_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	widget->allocation = *allocation;
 	bvw->priv->xpos = allocation->x;
 	bvw->priv->ypos = allocation->y;
-	
+
 	if ( (bvw->priv->init_width == 0) && (bvw->priv->init_height == 0) ) {
 		/* First allocation, saving values */
 		bvw->priv->init_width = allocation->width;
@@ -2232,17 +2241,71 @@ bacon_video_widget_get_visuals_list (BaconVideoWidget *bvw)
 gboolean
 bacon_video_widget_set_visuals (BaconVideoWidget *bvw, const char *name)
 {
+	xine_post_t *newvis;
+
 	g_return_val_if_fail (bvw != NULL, FALSE);
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 
-	g_free (bvw->priv->vis_name);
-	bvw->priv->vis_name = g_strdup (name);
-
-	if (GTK_WIDGET_REALIZED (GTK_WIDGET (bvw)))
-		return TRUE;
-	else
+	if (bvw->priv->null_out != FALSE || bvw->priv->ao_driver == NULL)
 		return FALSE;
+
+	if (GTK_WIDGET_REALIZED (bvw) == FALSE)
+	{
+		g_free (bvw->priv->vis_name);
+		bvw->priv->vis_name = g_strdup (name);
+		return FALSE;
+	}
+
+	newvis = xine_post_init (bvw->priv->xine,
+			name, 0,
+			&bvw->priv->ao_driver, &bvw->priv->vo_driver);
+
+	if (newvis != NULL)
+	{
+		g_free (bvw->priv->vis_name);
+		bvw->priv->vis_name = g_strdup (name);
+
+		if (bvw->priv->vis != NULL)
+		{
+			xine_post_t *oldvis;
+
+			oldvis = bvw->priv->vis;
+			bvw->priv->vis = newvis;
+
+			if (bvw->priv->using_vfx == TRUE)
+			{
+				//FIXME kludgy hack to avoid locking up
+				//when switching plugins and paused
+#if 1
+				Speeds speed;
+
+				speed = bacon_video_widget_get_speed (bvw);
+				if (speed == SPEED_PAUSE)
+				{
+					xine_set_param (bvw->priv->stream,
+							XINE_PARAM_SPEED,
+							XINE_SPEED_SLOW_4);
+				}
+#endif
+				show_vfx_update (bvw, FALSE);
+				show_vfx_update (bvw, TRUE);
+#if 1
+				if (speed == SPEED_PAUSE)
+				{
+					bacon_video_widget_set_speed (bvw,
+							SPEED_PAUSE);
+				}
+#endif
+			}
+
+			xine_post_dispose (bvw->priv->xine, oldvis);
+		} else {
+			bvw->priv->vis = newvis;
+		}
+	}
+
+	return FALSE;
 }
 
 void
@@ -2584,13 +2647,13 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		BaconVideoWidgetAudioOutType type)
 {
 	xine_cfg_entry_t entry;
-	int four_channel, five_channel, five_one_channel;
+	int four_channel, five_channel, five_one_channel, passthru;
 
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
-	four_channel = five_channel = five_one_channel = 0;
+	four_channel = five_channel = five_one_channel = passthru = 0;
 
 	gconf_client_set_int (bvw->priv->gc,
 			GCONF_PREFIX"/audio_output_type",
@@ -2608,6 +2671,9 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 	case BVW_AUDIO_SOUND_51CHANNEL:
 		five_one_channel = 1;
 		break;
+	case BVW_AUDIO_SOUND_AC3PASSTHRU:
+		passthru = 1;
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -2629,6 +2695,13 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		entry.num_value = five_one_channel;
 		xine_config_update_entry (bvw->priv->xine, &entry);
 	}
+
+	if (xine_config_lookup_entry (bvw->priv->xine, "audio.a52_pass_through",
+ &entry))
+	{
+		entry.num_value = passthru;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
 }
 
 static void
