@@ -291,12 +291,17 @@ static void
 on_button1_clicked (GtkButton *button, Totem *totem)
 {
 	GError *err = NULL;
-	char *path, *cmd;
+	char *path, *cmd, *filemanager;
 
 	path = path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
 
-	cmd = g_strdup_printf ("nautilus --no-default-window %s", path);
+	if (g_getenv ("KDE_STARTUP_ENV") && (filemanager = g_find_program_in_path("konqueror"))) {
+		cmd = g_strconcat (filemanager, " ", path, NULL);
+		g_free (filemanager);
+	} else {
+		cmd = g_strdup_printf ("nautilus --no-default-window -no-desktop %s", path);
+	}
 	g_free (path);
 
 	if (g_spawn_command_line_async (cmd, &err) == FALSE)
@@ -563,6 +568,12 @@ totem_setup_preferences (Totem *totem)
 			totem, NULL, NULL);
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_deinterlace1_activate), totem);
+
+	/* Always On Top */
+	item = glade_xml_get_widget (totem->xml, "tmw_always_on_top_menu_item");
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+			gconf_client_get_bool (totem->gc,
+				GCONF_PREFIX"/window_on_top", NULL));
 }
 
 void
@@ -620,3 +631,4 @@ totem_preferences_visuals_setup (Totem *totem)
 
 	bacon_video_widget_set_visuals (totem->bvw, visual);
 }
+
